<?php
/** 
* @file PortaMx_AdminBlocksClass.php
* Global Blocks Admin class
*
* @author PortaMx - Portal Management Extension
* @author Copyright 2008-2009 by PortaMx - http://portamx.com
* @Version: 0.957
* @Date: 11.03.2009
*/

if(!defined('PortaMx'))
	die('This file can\'t be run without PortaMx');

/**
* @class PortaMxC_AdminBlocks
* The Global Class for Block Administration.
* @see PortaMx_AdminBlocksClass.php
* @author Copyright by PortaMx - http://portamx.com
*/
class PortaMxC_AdminBlocks
{
	var $cfg;						///< common config

	/**
	* The Contructor.
	* Saved the config, load the block css and language file if exist.
	* Have the block a css file, the class definition is extracted from ccs header
	*/
	function PortaMxC_AdminBlocks($blockconfig)
	{
		global $context;

		// have a xmlset, convert to array
		if(isset($blockconfig['config']))
			$blockconfig['config'] = unserialize($blockconfig['config']);
		$this->cfg = $blockconfig;

		// load the block css file if exist
		if(file_exists($context['PMx_syscssdir'] . $this->cfg['blocktype'] .'.css'))
		{
			if(strpos($context['html_headers'], $this->cfg['blocktype'] .'.css') === false)
				$context['html_headers'] .= ' 
	<link rel="stylesheet" type="text/css" href="'. $context['PMx_syscssurl'] . $this->cfg['blocktype'] .'.css" />';
		}

		// have a css file, load and get classdefs
		if(!empty($this->cfg['config']['cssfile']))
		{
			// read the cssfile
			$css = file_get_contents($context['PMx_blockscssdir'] . $this->cfg['config']['cssfile'] .'.css');

			// get classdefs
			preg_match('~(<([\w]+)[^>]*>)(.*)(<\/\\2>)~s', $css, $match);
			if(isset($match[0]) && !empty($match[0]))
			{
				$tmp = ParseXml($match[0]);
				if($tmp->name == 'class')
				{
					foreach($tmp->childs as $data)
						$this->cfg['config']['cssfileclass'][$data->name] = $data->content;
				}
			}
		}

		// have a language file, load them
		if(!empty($this->cfg['config']['langfile']))
			loadLanguage($context['PMx_blockslangdir'] . $this->cfg['config']['langfile']);
	}

	/**
	* Get a config item.
	* The item can be empty, a single value or a array
	*/
	function pmxc_AdmBlock_getConfig($item = '')
	{
		$result = null;

		if(empty($item))								// no Item, get all
			$result = $this->cfg;
		elseif(!is_array($item))				// no array, get item
			$result = $this->cfg[$item];
		else														// array, find the item
		{
			$ptr = &$this->cfg;
			foreach($item as $key)
				$ptr = &$ptr[$key];

			if(isset($ptr))
				$result = $ptr;
		}
		return $result;
	}

	/**
	* Check access for Block moderation
	* if checkacs true, the block mod acs is checked is not in page moderate.
	*/
	function pmxc_ModAllowed($checkacs = false)
	{
		global $context, $user_info;

		$access = AllowedTo('admin_forum');
		if(!$access)
		{
			if(allowedTo('manage_portamx') && isset($context['PortaMx']['settings']['manager']['admin_pages']) && in_array($this->cfg['side'], $context['PortaMx']['settings']['manager']['admin_pages']))
				$access = true;
			elseif(allowedTo('manage_portamx') && $checkacs)
			{
				foreach($user_info['groups'] as $g)
					$access = (isset($this->cfg['config']['mod_acs']) && in_array($g, $this->cfg['config']['mod_acs']) ? true : $access);
			}
		}
		return $access;
	}
}

/**
* @class pmxc_AdmBlockOverview
* This class prepare the Block overview screen.
* In this you see all settings and you can add, edit and remove a block from here.
* @see PortaMx_AdminBlocksClass.php
* @author Copyright by PortaMx - http://portamx.com
*/
class pmxc_AdmBlockOverview extends PortaMxC_AdminBlocks
{
	var $reg_blocks;			///< contains all registered blocks

	/**
	* Called from AdminBlocks template 
	*/
	function pmxc_ShowAdmBlockOverview($side, $bcount)
	{
		global $context, $user_info, $txt;

		$this->reg_blocks = $context['PortaMx']['RegisterBlocks'];
		$LtR = empty($context['right_to_left']) ? 'left' : 'right';
		$RtL = empty($context['right_to_left']) ? 'right' : 'left';

		$extOpts = false;
		if(isset($this->cfg['config']['ext_opts']))
		{
			foreach($this->cfg['config']['ext_opts'] as $k => $v)
				$extOpts = (isset($v) && !empty($v) ? true : $extOpts);
		}

		if($this->pmxc_ModAllowed(true))
		{
			echo '
				<tr class="windowbg" style="height:20px;">
					<td valign="middle" style="padding:3px 5px;white-space:nowrap;">
						<input style="width:28px;" onkeyup="check_numeric(this)" type="text" size="2" name="pos['. $this->cfg['id'] .']" value="'. $this->cfg['pos'] .'"'.($this->pmxc_ModAllowed() ? '' : ' disabled="disabled"') .' />';

			if($this->pmxc_ModAllowed())
			{
				$pstat = explode(':', $bcount);
				if(!empty($pstat[0]))
					echo '
						<img style="cursor:pointer;" src="'. $context['PMx_imageurl'] .'down.gif" alt="down" title="'. $txt['move_down'] .'" onclick="FormFunc(\'move_down\', \''. $this->cfg['id'] .'\')" />';
				else
					echo '
						<img src="'. $context['PMx_imageurl'] .'empty.gif" alt="*" title="" />';
				if(!empty($pstat[1]))
					echo '
						<img style="cursor:pointer;" src="'. $context['PMx_imageurl'] .'up.gif" alt="up" title="'. $txt['move_up'] .'" onclick="FormFunc(\'move_up\', \''. $this->cfg['id'] .'\')" />';
				else
					echo '
						<img src="'. $context['PMx_imageurl'] .'empty.gif" alt="*" title="" />';
			}
			else
				echo '
						<img width="34" height="16" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" title="" />';

			echo '
					</td>
					<td valign="middle" align="center" style="padding:3px 5px;">
						<img src="'. $context['PMx_imageurl'] .'text_align_'. (isset($this->cfg['config']['title_align']) ? $this->cfg['config']['title_align'] : 'left') .'.gif" alt="*" title="'. $txt['pxm_edit_title_align'] .(isset($this->cfg['config']['title_align']) ? ' '. $txt['pxm_edit_title_align_types'][$this->cfg['config']['title_align']] : ' '. $txt['pxm_edit_title_align_types']['left']). '" />
					</td>
					<td valign="middle" style="padding:3px 5px;">
						'. (!empty($this->cfg['config']['title_icon']) ? '<img align="'.$LtR.'" style="padding-'.$RtL.':4px;" src="'. $context['PMx_Iconsurl'] . $this->cfg['config']['title_icon'] .'" alt="*" title="'. substr($txt['pxm_edit_titleicon'], 0, -1) .'" />' : '') .'
						'. (isset($this->cfg['config']['title'][$user_info['language']]) ? $this->cfg['config']['title'][$user_info['language']] : '') .'
					</td>
					<td valign="middle" style="padding:3px 5px;">
						<img align="'.$LtR.'" style="padding-'.$RtL.':5px;" src="'. $context['PMx_imageurl'] .'type_'. $this->reg_blocks[$this->cfg['blocktype']]['icon'] .'.gif" alt="*" title="'. $this->reg_blocks[$this->cfg['blocktype']]['blocktype'] .' '. $this->reg_blocks[$this->cfg['blocktype']]['description'].'" />'. $this->reg_blocks[$this->cfg['blocktype']]['description'] .'
					</td>
					<td align="center" valign="middle" style="padding:3px 5px;">
						<img src="'. $context['PMx_imageurl'] .'page'. ($this->cfg['active'] ? '_unlock' : '_lock') .'.gif" alt="*" title="'. ($this->cfg['active'] ? $txt['pxm_status_activ'] : $txt['pxm_status_inactiv']) . ($this->pmxc_ModAllowed(true) ? ' - '. $txt['pxm_status_change'] : '') .'" style="cursor:pointer;"'. ($this->pmxc_ModAllowed(true) ? ' onclick="FormFunc(\'chg_status\', \''. $this->cfg['id'] .'\')"' : '') .' />
					</td>
					<td valign="middle" style="padding:3px 0px 3px 2px;">
						<div id="options'. $this->cfg['id'] .'">
							'. (!empty($this->cfg['config']['settings']) ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'settings.gif" alt="*" title="'. $txt['pmx_have_settings'] .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
							'. (!empty($this->cfg['config']['grp_acs']) ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'group.gif" alt="*" title="'. $txt['pmx_have_groupaccess'] .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
							'. (!empty($this->cfg['config']['mod_acs']) ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'user.gif" alt="*" title="'. $txt['pmx_have_modaccess'] .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
							'. ($extOpts ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'app_dyn.gif" alt="*" title="'. $txt['pmx_have_dynamics'] .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
							'. (!empty($this->cfg['config']['cssfile']) ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'css.gif" alt="*" title="'. $txt['pmx_have_cssfile'] .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
							'. (!empty($this->cfg['config']['langfile']) ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'langfile.gif" alt="*" title="'. $txt['pmx_have_langfile'] .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
							'. (!empty($this->cfg['cache']) ? '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'cache_refresh.gif" alt="*" title="'. $txt['pmx_have_caching'] .' ('. $this->cfg['cache'] .' Min)' .'" />' : '<img style="padding:0px 2px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" />') .'
						</div>
						<div class="smalltext" id="move'. $this->cfg['id'] .'" style="text-align:'.$RtL.';width:165px;display:none;">
							'. $txt['pxm_moveto'] .'
							<select style="width:55px;" size="1" name="" onchange="setBlockSide(this, \''. $this->cfg['id'] .'\')">';

			foreach($txt['pmx_block_sides'] as $moveside => $sidetext)
				echo '
								<option value="'. $moveside .'"'. ($moveside == $side ? ' selected="selected"' : '') .'>'. $sidetext .'</option>';

			echo '
							</select>
						</div>
					</td>
					<td valign="middle" style="padding:3px 5px;" nowrap="nowrap">
						'. ($this->pmxc_ModAllowed(true) ? '<img style="padding:0px 1px; cursor:pointer;" src="'. $context['PMx_imageurl'] .'page_edit.gif" alt="*" title="'. $txt['pxm_edit_sideblock'] .'" onclick="FormFunc(\'edit_block\', \''. $this->cfg['id'] .'\')" />' : '<img style="padding:0px 1px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" title="" />') .'
						'. (allowedTo('admin_forum') ? '<img style="padding:0px 1px; cursor:pointer;" src="'. $context['PMx_imageurl'] .'page_move.gif" alt="*" title="'. $txt['pxm_move_sideblock'] .'" onclick="ToggleMove(\''. $this->cfg['id'] .'\')" />' : '<img style="padding:0px 1px;" src="'. $context['PMx_imageurl'] .'empty.gif" alt="" title="" />') .'
						'. ($this->pmxc_ModAllowed() ? '<img style="padding:0px 2px; cursor:pointer;" src="'. $context['PMx_imageurl'] .'page_del.gif" alt="*" title="'. $txt['pxm_delete_sideblock'] .'" onclick="FormFunc(\'block_delete\', \''. $this->cfg['id'] .'\', \''. $txt['pmx_confirm_blockdelete'] .'\')" />' : '');
			echo '
					</td>
				</tr>';
		}
	}
}

/**
* @class PortaMxC_SystemAdminBlock
* This is the Global Admin class to create or edit a single Block.
* This class prepare the settings screen and call the user dependent settings and content classes.
* @see PortaMx_AdminBlocksClass.php
* @author Copyright by PortaMx - http://portamx.com
*/
class PortaMxC_SystemAdminBlock extends PortaMxC_AdminBlocks
{
	var $smf_langs;					///< all languages
	var $blockcssfiles;			///< all block cssfiles
	var $smf_groups;				///< all usergroups
	var $smf_boards;				///< all smf boards
	var $register_blocks;		///< all registered blocks
	var $block_classdef;		///< all default classes
	var $can_cached;				///< block can cached (1), not cached (0)
	var $title_icons;				///< array with title icons

  /**
  * Get Config data from array with name=value format
  */
	function getConfigData($elm, $child)
	{
		$result = array();
		if(!empty($this->cfg['config'][$elm][$child]))
		{
			foreach($this->cfg['config'][$elm][$child] as $val)
			{
				$tmp = preg_split('~\=~', $val, -1, PREG_SPLIT_NO_EMPTY);
				if(isset($tmp[0]) && isset($tmp[1]))
					$result[$tmp[0]] = $tmp[1];
			}
		}
		return $result;
	}
  
	/**
	* Output the Block config screen
	*/
	function pmxc_ShowAdmBlockConfig()
	{
		global $context, $settings, $options, $txt;

		if(!$this->pmxc_ModAllowed(true))
		{
			echo '
					</table>
				</td>
			</tr>
		</table>';
			isAllowedTo('admin_forum');
		}

		// directions
		$isIE = $context['browser']['is_ie'];
		$LtR = empty($context['right_to_left']) ? 'left' : 'right';
		$RtL = empty($context['right_to_left']) ? 'right' : 'left';
		$floatL = ' style="float:' .(empty($context['right_to_left']) ? 'left' : 'right'). ';"';

		echo '
				<tr class="windowbg">
					<td valign="top" width="50%" style="padding:8px 4px;">
						<input type="hidden" name="config[settings]" value="" />
						<input id="curlang" type="hidden" name="" value="'. $context['user']['language'] .'" />
            <input type="hidden" name="check_num_vars[]" value="[config][maxheight], \'\'" />
            <input type="hidden" name="check_num_vars[]" value="[config][innerpad], \'4\'" />
						<div style="height:50px;">
							<div style="float:'. $LtR .';width:110px; padding-top:1px;">'. $txt['pxm_edit_title'] .'</div>';

		// all titles depend on language
		foreach($this->smf_langs as $lang)
			echo '
							<span id="'. $lang .'" style="white-space:nowrap;'. ($context['user']['language'] == $lang ? '' : ' display:none;') .'">
								<input style="width:60%;" type="text" name="config[title]['. $lang .']" value="'. (isset($this->cfg['config']['title'][$lang]) ? htmlspecialchars($this->cfg['config']['title'][$lang], ENT_QUOTES) : '') .'" />
							</span>';

		echo '
							<div style="clear:both; height:10px;">
								<img style="float:'. $LtR .';" src="'. $context['PMx_imageurl'] .'arrow_down.gif" alt="*" title="" />
							</div>
							<div style="float:'. $LtR .'; width:110px;">'. $txt['pxm_edit_title_lang'] .'
								<img class="info_toggle" onclick=\'Show_help("pmxBH01")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
							</div>
							<select style="float:'. $LtR .'; width:26%;" size="1" name="" onchange="setTitleLang(this)">';

		foreach($this->smf_langs as $lang)
			echo '
								<option value="'. $lang .'"' .($context['user']['language'] == $lang ? ' selected="selected"' : '') .'>'. $lang .'</option>';
		echo '
							</select>
							<div style="float:'. $LtR .'; padding-' . $LtR .':15px;"><span style="vertical-align:6px;">'. $txt['pxm_edit_title_align'] .'</span>';

		// title align
		if(!isset($this->cfg['config']['title_align']))
			$this->cfg['config']['title_align'] = 'left';

		echo '
							<input type="hidden" id="titlealign" name="config[title_align]" value="'. $this->cfg['config']['title_align'] .'" />';

		foreach($txt['pxm_edit_title_align_types'] as $key => $val)
			echo '
								<img id="img'. $key .'" class="tborder" src="'. $context['PMx_imageurl'] .'text_align_'. $key .'.gif" alt="*" title="'. $txt['pxm_edit_title_helpalign']. $val .'" style="cursor:pointer;'.($this->cfg['config']['title_align'] == $key ? ' background-color:#e02000;' : '').'" onclick="setAlign(\''. $key .'\')" />';

		echo '
							</div>
							<div class="adm_clear" style="line-height:1px;">&nbsp;</div>
						</div>
						<div id="pmxBH01" class="info_frame">'. 
							$txt['pxm_edit_titlehelp'] .'
						</div>';

		// Title icon
		echo '
						<div class="adm_clear">
							<div style="float:'. $LtR .';width:110px; padding-top:5px;">'. $txt['pxm_edit_titleicon'] .'</div>
							<select id="init_ttlicon" style="width:55%; margin-top:5px;" size="1" name="config[title_icon]" onchange="setTitleIcon(this);">
								<option value=""' .(empty($this->cfg['config']['title_icon']) ? ' selected="selected"' : '') .'>'. $txt['pxm_edit_no_icon'] .'</option>';

		foreach($this->title_icons as $name)
			echo '
								<option value="'. $name .'"' .(!empty($this->cfg['config']['title_icon']) && $this->cfg['config']['title_icon'] == $name ? ' selected="selected"' : '') .'>'. $name .'</option>';
		echo '
							</select>
							<img src="'. $context['PMx_imageurl'] .'empty.gif" width="5" alt="*" title="" />
							<img style="padding-top:7px; vertical-align:top;" id="pmxttlicon" src="'. $context['PMx_Iconsurl'] .'none.gif" alt="" title="" />
							<script type="text/javascript"><!-- // --><![CDATA[
								setTitleIcon(document.getElementById("init_ttlicon"));
							// ]]></script>
						</div>';

		// show registered block types
		echo '
					</td>
					<td valign="top" style="padding:8px 4px;">
						<div style="height:50px;">
							<div style="float:'. $LtR .'; width:120px; padding-top:2px;">'. $txt['pxm_edit_type'] .'</div>';

		if(!$this->pmxc_ModAllowed())
			echo '
							<input type="hidden" name="blocktype" value="'. $this->cfg['blocktype'] .'" />
							<input style="width:50%;" type="text" name="blocktype" value="'. $this->cfg['blocktype'] .'" disabled="disabled" />';
		else
		{
			echo '
							<select style="width:50%;" size="1" name="blocktype" onchange="FormFunc(\'edblock_blocktypechg\', \''. $this->cfg['id'] .'\')">';

			foreach($this->register_blocks as $blocktype => $blockDesc)
				echo '
								<option value="'. $blocktype .'"' .($this->cfg['blocktype'] == $blocktype ? ' selected="selected"' : '') .'>'. $blockDesc['description'] .'</option>';
			echo '
							</select>';
		}

		// cache settings
		echo '
							<input type="hidden" name="cache" value="0" />
							<div style="clear:both; line-height:7px;">&nbsp;</div>
							<div style="float:'. $LtR .'; width:117px; padding-top:5px;">'. $txt['pxm_edit_cache'] .'
							</div>
							<input style="float:'. $LtR .'; margin-top:'.($isIE ? '5' : '7').'px;" id="cacheflag" class="check" type="checkbox" name="cacheflag" onclick="checkCache(this)" value="1"'.(!empty($this->can_cached) ? (!empty($this->cfg['cache']) ? ' checked="checked"' : '') : ' disabled="disabled"'). ' />

							<div style="float:'. $LtR .'; margin-'. $LtR .':15px; padding-top:5px;">'. $txt['pxm_edit_cachetime'] .'</div>
							<input style="float:'. $LtR .'; margin-top:3px; margin-'. $LtR .':6px;" onkeyup="check_numeric(this, \'*\')" id="cacheval" type="text" name="cache" value="'.(empty($this->can_cached) ? '0" disabled="disabled' : (!empty($this->cfg['cache']) ? $this->cfg['cache'] : '0" disabled="disabled')) .'" size="7" />
							<div class="smalltext" style="float:'. $LtR .'; margin-'. $LtR .':3px; padding-top:5px;">'. $txt['pxm_edit_cachetimemin'] .'
								<img class="info_toggle" onclick=\'Show_help("pmxBH02")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
							</div>
							<div style="clear:both; line-height:4px; margin-top:-4px;">&nbsp;</div>
						</div>

						<div id="pmxBH02" class="info_frame" style="margin-top:5px">'.
							(!empty($this->can_cached) ? $txt['pxm_edit_cachehelp'] : $txt['pxm_edit_nocachehelp']) .'
						</div>';

			// Pagename
		if($this->cfg['side'] == 'pages')
			echo '
						<div class="adm_clear">
							<div class="adm_clear" style="float:'. $LtR .';width:120px; padding-top:7px;">'. $txt['pxm_edit_pagename'] .'
								<img class="info_toggle" onclick=\'Show_help("pmxBH11")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
							</div>
							<input style="width:50%; margin-top:5px;" type="text" name="config[pagename]" value="'. (!empty($this->cfg['config']['pagename']) ? $this->cfg['config']['pagename'] : '') .'" />
						</div>

						<div id="pmxBH11" class="info_frame" style="margin-top:5px;">'.
							$txt['pxm_edit_pagenamehelp'] .'
						</div>';
		else
			echo '
						<input type="hidden" name="config[pagename]" value="'. (!empty($this->cfg['config']['pagename']) ? $this->cfg['config']['pagename'] : '') .'" />';

		echo '
					</td>
				</tr>
				<tr class="windowbg">';

		/*
		* Call the block depended settings.
		* Because each block can have his own settings, we have to call the settings now.
		*/
		$usedClass = $this->pmxc_AdmBlock_content();

		// the visual options
		echo '
					<td id="set_col" valign="top" style="padding:4px;">
						<div class="titlebg" style="padding:2px; margin-bottom:4px;">'. $txt['pmx_edit_visuals'] .'</div>
						<div style="float:'. $LtR .'; height:22px; width:117px;">'. $txt['pxm_edit_cancollapse'] .'</div>
						<input style="padding-'. $LtR .':120px;" type="hidden" name="config[collapse]" value="0" />
						<input class="check" id="collapse" type="checkbox" name="config[collapse]" value="1"'. ($this->cfg['config']['visuals']['header'] == 'none' ? ' disabled="disabled"' : ($this->cfg['config']['collapse'] == 1 ? ' checked="checked"' : '')) .' />
						<br style="clear:both;" />
						<div style="float:'. $LtR .'; height:22px; width:120px;">'. $txt['pxm_edit_maxheight'] .'</div>
						<input onkeyup="check_numeric(this)" id="maxheight" type="text" size="4" name="config[maxheight]" value="'. (isset($this->cfg['config']['maxheight']) ? $this->cfg['config']['maxheight'] : '') .'"'. (!isset($this->cfg['config']['overflow']) || empty($this->cfg['config']['overflow']) ? ' disabled="disabled"' : '') .' /><span class="smalltext">'. $txt['pmx_pixel_blank'] .'</span>
						<br style="clear:both;" />
						<div style="float:'. $LtR .'; height:22px; width:120px;">'. $txt['pxm_edit_overflow'] .'</div>
						<select style="width:50%;" size="1" id="mxhgt" name="config[overflow]" onchange="checkMaxHeight(this);">';

		foreach($txt['pmx_overflow_actions'] as $key => $text)
			echo '
							<option value="'. $key .'"'. (isset($this->cfg['config']['overflow']) && $this->cfg['config']['overflow'] == $key ? ' selected="selected"' : '') .'>'. $text .'</option>';
		echo '
						</select>
						<br style="clear:both;" />

						<script type="text/javascript"><!-- // --><![CDATA[
							checkMaxHeight(document.getElementById("mxhgt"));
						// ]]></script>

						<div style="float:'. $LtR .'; height:22px; width:120px;">'. $txt['pxm_edit_innerpad'] .'</div>
						<input onkeyup="check_numeric(this)" type="text" size="3" name="config[innerpad]" value="'. (isset($this->cfg['config']['innerpad']) ? $this->cfg['config']['innerpad'] : '4') .'" /><span class="smalltext">'. $txt['pmx_pixel'] .'</span>
						<br style="clear:both;" />';

		// CSS class settings
		echo '
						<div class="titlebg2" style="padding:2px;margin:4px 0px;">
							<div style="float:'. $LtR .'; width:120px;">'. $txt['pmx_edit_usedclass_type'] .'</div>
							'. $txt['pmx_edit_usedclass_style'] .'
						</div>
						<div style="margin:0px 2px;">';

		// have a user css file ?
		if(!empty($this->cfg['config']['cssfile']) && isset($this->cfg['config']['cssfileclass']))
      $userclass = array_keys($this->cfg['config']['cssfileclass']);
		else
      $userclass = array();

		// write out the default classes
		foreach($usedClass as $ucltyp => $ucldata)
		{
			// skip if in userclass
			if(!in_array($ucltyp, $userclass))
			{
				echo '
							<div style="float:'. $LtR .'; width:120px; height:20px; padding-top:2px;">'. $ucltyp .'</div>
							<select style="width:50%;" name="config[visuals]['. $ucltyp .']" onchange="checkCollapse(this)">';

					foreach($ucldata as $cname => $class)
						echo '
								<option value="'. $class .'"'. (isset($this->cfg['config']['visuals'][$ucltyp]) ? ($this->cfg['config']['visuals'][$ucltyp] == $class ? ' selected="selected"' : '') : (substr($cname,0,1) == '+' ? ' selected="selected"' : '')) .'>'. substr($cname, 1) .'</option>';

					echo '
							</select>
							<br style="clear:both;" />';
			}
		}

		// write out the user classes
		foreach($userclass as $cname)
			echo '
							<div style="float:'. $LtR .'; width:120px; height:20px; padding-top:2px;">'. $cname .'</div>
							<input type="hidden" name="config[visuals]['. $cname .']" value="'. $this->cfg['config']['cssfileclass'][$cname] .'" />
							'. $this->cfg['config']['cssfileclass'][$cname] .'<br style="clear:both;" />';

		echo '
						</div>
						<div class="titlebg2" style="padding:2px;margin:4px 0px;">'. $txt['pmx_edit_canhavecssfile'] .'</div>
						<div style="float:'. $LtR .'; margin:0px 2px; width:116px;">'. $txt['pxm_edit_cssfilename'] .'</div>
						<select style="width:50%;margin-bottom:2px;" name="config[cssfile]" onchange="FormFunc(\'edblock_ccsfilechg\', \''. $this->cfg['id'] .'\')">';

		// show all exist user css files
		foreach($this->blockcssfiles as $cssfile)
			echo '
							<option value="'. $cssfile .'"'. ($this->cfg['config']['cssfile'] == $cssfile ? ' selected="selected"' : '') .'>'. $cssfile .'</option>';
		echo '
						</select>';
		echo '
					</td>
				</tr>
				<tr class="windowbg">';

		// the group access
		echo '
					<td valign="top" style="padding:4px;">
						<div class="titlebg" style="padding:2px;margin-bottom:2px;">'. $txt['pmx_edit_groups'] .'</div>
						<img class="info_toggle" align="'. $RtL .'" style="padding:5px;"; onclick=\'Show_help("pmxBH03")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
						<select style="margin:5px 0px;width:90%;" name="config[grp_acs][]" multiple="multiple" size="5">';

		foreach($this->smf_groups as $grp)
			echo '
							<option value="'. $grp['id'] .'"'. (!empty($this->cfg['config']['grp_acs']) && in_array($grp['id'], $this->cfg['config']['grp_acs']) ? ' selected="selected"' : '') .'>'. $grp['name'] .'</option>';
		echo '
						</select>

						<div id="pmxBH03" class="info_frame">'. $txt['pmx_edit_groups_help'] .'</div>
					</td>';

		// the moderator access
		echo '
					<td valign="top" style="padding:4px;">
						<div class="titlebg" style="padding:2px;margin-bottom:2px;">'. $txt['pmx_edit_modgroups'] .'</div>
						<div'. (allowedTo('admin_forum') ? '' : ' style="display:none;"') .'>
							<img class="info_toggle" align="'. $RtL .'" style="padding:5px;"; onclick=\'Show_help("pmxBH04")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
							<select style="margin:5px 0px;width:90%;" name="config[mod_acs][]" multiple="multiple" size="5">';

		foreach($this->smf_groups as $grp)
		{
			if($grp['id'] > 1)
				echo '
								<option value="'. $grp['id'] .'"'. (!empty($this->cfg['config']['mod_acs']) && in_array($grp['id'], $this->cfg['config']['mod_acs']) ? ' selected="selected"' : '') .'>'. $grp['name'] .'</option>';
		}
		echo '
							</select>

							<div id="pmxBH04" class="info_frame">'. (allowedTo('admin_forum') ? $txt['pmx_edit_modgroups_help'] : $txt['pmx_edit_modgroups_off'] ) .'</div>
						</div>
					</td>
				</tr>';

		// the dynamic visibility options
		$extOpts = 1;
		if(!empty($this->cfg['config']['ext_opts']))
			foreach($this->cfg['config']['ext_opts'] as $k => $v)
				$extOpts = (isset($v) && !empty($v) ? 0 : $extOpts);
		$options['collapse_visual'] = $context['PortaMx']['settings']['manager']['collape_visibility'] && $extOpts;

		echo '
				<tr class="windowbg">
					<td valign="top" colspan="2" style="padding:4px;">
						<div class="titlebg" style="padding:2px; text-align:center; margin-bottom:4px;">
							<div onclick="upshrinkVis.toggle(); setBlockUpshrinkTitle(\'upshrinkImgVisual\', \''. $txt['pmx_edit_ext_opts'] .'\');" style="cursor:pointer;">
								<img id="upshrinkImgVisual" align="left" style="padding-top:1px; padding:0 5px;" src="'. $context['PMx_imageurl'] .(empty($options['collapse_visual']) ? '/collapse' : '/expand'). '.gif" alt="*" title="'. (empty($options['collapse_visual']) ? $txt['pmx_collapse'] : $txt['pmx_expand']) . $txt['pmx_edit_ext_opts'] .'" />
							</div>'. $txt['pmx_edit_ext_opts'] .'
						</div>
						<div id="upshrinkVisual1"' .(empty($options['collapse_visual']) ? '' : ' style="display:none;"') .'>

							<div class="info_border tborder" style="margin-top:5px;">
								<span><img onclick=\'Toggle_help("pmxBH05")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" /></span>
								<div>'. $txt['pmx_edit_ext_opts_help'] .'</div>
								<div id="pmxBH05" style="margin-top:-16px; display:none;">'. $txt['pmx_edit_ext_opts_morehelp'] .'</div>
							</div>

						</div>
					</td>
				</tr>
				<tr class="windowbg" id="upshrinkVisual"' .(empty($options['collapse_visual']) ? '' : ' style="display:none;"'). '>';

		// on default actions
		echo '
					<td valign="top" style="padding:4px;">
						<div class="titlebg2" style="padding:2px; margin-bottom:2px;">'. $txt['pmx_edit_ext_opts_action'] .'</div>
						<img class="info_toggle" align="'. $RtL .'" style="padding:5px;"; onclick=\'Show_help("pmxBH06")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
						<select id="pmxact" onchange="changed(\'pmxact\');" style="margin:5px 0px;width:90%;" name="config[ext_opts][pmxact][]" multiple="multiple" size="5">';

		// get config data
		$data = $this->getConfigData('ext_opts', 'pmxact');
		foreach($txt['pmx_action_names'] as $act => $actdesc)
			echo '
							<option value="'. $act .'='. (array_key_exists($act, $data) ? $data[$act] .'" selected="selected' : '1') .'">'. (array_key_exists($act, $data) ? ($data[$act] == 0 ? '^' : '') : '') . $actdesc .'</option>';
 
		echo '
						</select>

						<div id="pmxBH06" class="info_frame">'. $txt['pmx_edit_ext_opts_selnote'] .'</div>

						<script type="text/javascript"><!-- // --><![CDATA[
							var pmxact = new MultiSelect("pmxact");
						// ]]></script>
					</td>';

		echo '
					<td valign="top" style="padding:4px;">
						<div class="titlebg2" style="padding:2px; margin-bottom:2px;">'. $txt['pmx_edit_ext_opts_custaction'] .'</div>
						<div style="padding:5px 0px;">
							'. $txt['pmx_edit_ext_opts_custom'] .'
							<img class="info_toggle" onclick=\'Show_help("pmxBH07")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
						</div>
						<input style="width:95%;" type="text" name="config[ext_opts][pmxcust]" value="'. $this->cfg['config']['ext_opts']['pmxcust'] .'" />

						<div id="pmxBH07" class="info_frame" style="margin-top:5px;">'. $txt['pmx_edit_ext_opts_custhelp'] .'</div>
					</td>
				</tr>
				<tr class="windowbg" id="upshrinkVisual2"' .(empty($options['collapse_visual']) ? '' : ' style="display:none;"'). '>';

		// on boards
		echo '
					<td valign="top" style="padding:4px;">
						<div class="titlebg2" style="padding:2px;margin-bottom:2px;">'. $txt['pmx_edit_ext_opts_boards'] .'</div>
						<img class="info_toggle" align="'. $RtL .'" style="padding:5px;"; onclick=\'Show_help("pmxBH08")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
						<select id="pmxbrd" onchange="changed(\'pmxbrd\');" style="margin:5px 0px; width:90%;" name="config[ext_opts][pmxbrd][]" multiple="multiple" size="5">';

		// add any at top of boards
		$boards = array_merge(array(array('id' => 'any', 'name' => $txt['pmx_edit_ext_opts_any'])), $this->smf_boards);
		// get config data
		$data = $this->getConfigData('ext_opts', 'pmxbrd');
		foreach($boards as $brd)
			echo '
							<option value="'. $brd['id'] .'='. (array_key_exists($brd['id'], $data) ? $data[$brd['id']] .'" selected="selected' : '1') .'">'. (array_key_exists($brd['id'], $data) ? ($data[$brd['id']] == '0' ? '^' : '') : '') . $brd['name'] .'</option>';

		echo '
						</select>

						<div id="pmxBH08" class="info_frame">'. $txt['pmx_edit_ext_opts_selnote'] .'</div>

						<script type="text/javascript"><!-- // --><![CDATA[
							var pmxbrd = new MultiSelect("pmxbrd");
						// ]]></script>
					</td>';

		// on language
		echo '
					<td valign="top" style="padding:4px;">
						<div class="titlebg2" style="padding:2px;margin-bottom:2px;">'. $txt['pmx_edit_ext_opts_languages'] .'</div>
						<img class="info_toggle" align="'. $RtL .'" style="padding:5px;"; onclick=\'Show_help("pmxBH09")\' src="'. $context['PMx_imageurl'] .'information.gif" alt="*" title="'. $txt['pmx_information_icon'] .'" />
						<select id="pmxlng" onchange="changed(\'pmxlng\');" style="margin:5px 0px; width:90%;" name="config[ext_opts][pmxlng][]" multiple="multiple" size="5">';

		// add any at top of languages
		$languages = array_merge(array('any' => $txt['pmx_edit_ext_opts_any']), $this->smf_langs);
		// get config data
		$data = $this->getConfigData('ext_opts', 'pmxlng');
		foreach($languages as $lng => $ldesc)
			echo '
							<option value="'. $lng .'='. (array_key_exists($lng, $data) ? $data[$lng] .'" selected="selected' : '1') .'">'. (array_key_exists($lng, $data) ? ($data[$lng] == 0 ? '^' : '') : '') . $ldesc .'</option>';

		echo '
						</select>

						<div id="pmxBH09" class="info_frame">'. $txt['pmx_edit_ext_opts_selnote'] .'</div>

						<script type="text/javascript"><!-- // --><![CDATA[
							var pmxlng = new MultiSelect("pmxlng");
						// ]]></script>
					</td>
				</tr>';
	}

	/**
	* This Methode is called on loadtime.
	* After all variables initiated, it calls the block dependent init methode.
	* Finaly the css and language is loaded if exist
	*/
	function pmxc_AdmBlock_loadinit()
	{
		global $context;

		$this->smf_langs = PortaMx_getLanguages();											// get all languages
		$this->blockcssfiles = PortaMx_getAllCssFiles();								// get all block cssfiles
		$this->smf_groups = PortaMx_getUserGroups();										// get all usergroups
		$this->smf_boards = PortaMx_getsmfBoards();											// get all smf boards
		$this->register_blocks = $context['PortaMx']['RegisterBlocks'];	// get all registered block
		$this->block_classdef = PortaMx_getdefaultClass();							// get default classes
		$this->can_cached = 0;																					// default no caching

		// sort the registered blocks
		ksort($this->register_blocks, SORT_STRING);

		// call the blockdepend init methode
		$langfile = $this->pmxc_AdmBlock_init();

		// have a language file and isloaded?
		if(!empty($langfile))
		{
			if(empty($this->cfg['config']['langfile']) || $this->cfg['config']['langfile'] != $langfile)
			{
				$this->cfg['config']['langfile'] = $langfile;
				loadLanguage($context['PMx_blockslangdir'] . $this->cfg['config']['langfile']);
			}
		}

		// find all title icons
		$this->title_icons = array();
		if(is_dir($context['PMx_Iconsdir'])) 
		{
			if($dh = opendir($context['PMx_Iconsdir'])) 
			{
				while(($file = readdir($dh)) !== false)
				{
					if(!is_dir($file) && $file != 'none.gif' && $file != '..' && $file != '.')
						$this->title_icons[] = $file;
				}
			}
			closedir($dh);
			sort($this->title_icons, SORT_STRING);
		}
	}

	/**
	* The default init Methode.
	* returns the blocklanguage if exist.
	* Note: Most blocks overwrite this methode.
	*/
	function pmxc_AdmBlock_init()
	{
		$this->can_cached = 0;		// disable caching
		return '';
	}

	/**
	* The default content Methode.
	* returns the blocksettings.
	* Note: Most blocks overwrite this methode.
	*/
	function pmxc_AdmBlock_content()
	{
		// default .. no content
		return $this->pmxc_AdmBlock_settings();
	}

	/**
	* The default sttings Methode.
	* returns the block css class definition.
	* Note: Most blocks overwrite this methode.
	*/
	function pmxc_AdmBlock_settings()
	{
		global $context, $txt;

		// the default settings
		echo '
					<td valign="top" style="padding:4px;">
						<input type="hidden" name="config[settings]" value="" />
						<input type="hidden" name="config[langfile]" value="" />
						<div>
							<div class="titlebg" style="padding:2px;margin-bottom:4px;">'. $txt['pmx_defaultsettings_title'] .'</div>'.
							$txt['pmx_defaultsettings'] .'
						</div>
					</td>';

		// return the default classnames
		return $this->block_classdef;
	}
}
?>