<?php
/** 
* @file PortaMx_BlocksClass.php
* Global Blocks class
*
* @author PortaMx - Portal Management Extension
* @author Copyright 2008-2009 by PortaMx - http://portamx.com
* @Version: 0.957
* @Date: 11.03.2009
*/

if(!defined('PortaMx'))
	die('This file can\'t be run without PortaMx');

/**
* @class PortaMxC_Blocks
* The Global Blocks class.
* @see PortaMx_BlocksClass.php
* @author Copyright by PortaMx - http://portamx.com
*/
class PortaMxC_Blocks
{
	var $cfg;				///< common config
	var $visible;		///< visibility flag

	/**
	* The Contructor.
	* Saved the config and checks the visiblity acces.
	* If access true, it load the block css and language file if exist.
	* Finally the javascript code for the block upshrink is stored in the header.
	*/
	function PortaMxC_Blocks($blockconfig)
	{
		global $context, $options;

		// load the config
		if(isset($blockconfig['config']))
			$blockconfig['config'] = unserialize($blockconfig['config']);	
  	$this->cfg = $blockconfig;

		// check if the block visible
		$show = $this->_checkAccess();

		// check page request
		if($show && $this->cfg['side'] == 'pages')
		{
			$show = !empty($_REQUEST['action']);
			if($show)
			{
				// any extension (wildcard) ?
				if(substr($_REQUEST['action'], -1) == '.')
					$show = substr($this->cfg['config']['pagename'], 0, strlen($_REQUEST['action'])-2) == substr($_REQUEST['action'], 1, -1);

				// no, check the custom action elements
				elseif(!empty($this->cfg['config']['ext_opts']['pmxcust']))
				{
					$tmp = preg_split('~\,~', $this->cfg['config']['ext_opts']['pmxcust'], -1, PREG_SPLIT_NO_EMPTY);
					foreach($tmp as $pgn)
					{
						$pgn = trim($pgn);

						// if a page action or any?
						if(strpos($pgn, '@') !== false || strpos($pgn, 'any') !== false)
						{
							if($pgn{0} == '^')
								$pgn = substr($pgn, 1);
							if(substr($pgn, -1) == '.')
								$show = substr($_REQUEST['action'], 0, strlen($pgn)-1) == substr($pgn, 0, -1);
							else
								$show = $_REQUEST['action'] == $pgn;
						}
					}
				}
				// no wildcard, check pagename
				else
					$show = $this->cfg['config']['pagename'] == substr($_REQUEST['action'], 1);
			}
		}

		// check dynamic visibility options
		if($show)
			$show = $this->_checkExtOpts(true);

		// continue if the block visible
		if($show)
		{
			// have a language file, load them
			if(!empty($this->cfg['config']['langfile']))
				loadLanguage($context['PMx_blockslangdir'] . $this->cfg['config']['langfile']);

			// load the block css file if exist
			if(file_exists($context['PMx_syscssdir'] . $this->cfg['blocktype'] .'.css')) 
				$context['html_headers'] .= ' 
	<link rel="stylesheet" type="text/css" href="'. $context['PMx_syscssurl'] . $this->cfg['blocktype'] .'.css" />';

			// have a user cssfile, load
			if(!empty($this->cfg['config']['cssfile']))
				$context['html_headers'] .= ' 
	<link rel="stylesheet" type="text/css" href="'. $context['PMx_blockscssurl'] . $this->cfg['config']['cssfile'] .'.css" />';

			// if header or frame and can collaps?
			if(isset($this->cfg['config']['collapse']) && $this->cfg['config']['collapse'] == 1 && $this->cfg['config']['visuals']['header'] != 'none')
			{
				if ($context['user']['is_guest'])
					$options['collapse'. $this->cfg['id']] = !empty($_COOKIE['upshrink'. $this->cfg['id']]);

				// add shrink code to header
				$context['html_headers'] .= '
	<script language="JavaScript" type="text/javascript"><!-- // --><![CDATA[
		var '. $this->cfg['side'] .'block'. $this->cfg['id'] .' = new smfToggle("upshrink'. $this->cfg['id'] .'", '. (empty($options['collapse'. $this->cfg['id']]) ? 'false' : 'true') .');
		'. $this->cfg['side'] .'block'. $this->cfg['id'] .'.useCookie('. ($context['user']['is_guest'] ? 1 : 0) .');
		'. $this->cfg['side'] .'block'. $this->cfg['id'] .'.setOptions("collapse'. $this->cfg['id'] .'", "'. $context['session_id'] .'");
		'. $this->cfg['side'] .'block'. $this->cfg['id'] .'.addToggleImage("upshrinkImg'. $this->cfg['id'] .'", "'. $context['PMx_imageurl'] .'collapse.gif", "'. $context['PMx_imageurl'] .'expand.gif", true);
		'. $this->cfg['side'] .'block'. $this->cfg['id'] .'.addTogglePanel("upshrinkBlock'. $this->cfg['id'] .'");
	// ]]></script>';
			}
		}
		$this->visible = $show;
	}

	/**
	* check the visibility access by group
	*/
	function _checkAccess()
	{
		global $user_info;

		$show = AllowedTo('admin_forum') ? true : false;
		if(isset($this->cfg['config']['grp_acs']))
		{
			foreach($user_info['groups'] as $g)
				$show = (in_array($g, $this->cfg['config']['grp_acs']) ? true : $show);
		}
		return $show;
	}

	/**
	* check the moderation access by group
	* if admin_pages set, the group access is overwrite
	*/
	function _modAccess($checkacs = false)
	{
		global $context, $user_info;

		$access = AllowedTo('admin_forum');
		if(!$access)
		{
			if(allowedTo('manage_portamx') && isset($context['PortaMx']['settings']['manager']['admin_pages']) && in_array($this->cfg['side'], $context['PortaMx']['settings']['manager']['admin_pages']))
				$access = true;
			elseif(allowedTo('manage_portamx') && $checkacs)
			{
				foreach($user_info['groups'] as $g)
					$access = (isset($this->cfg['config']['mod_acs']) && in_array($g, $this->cfg['config']['mod_acs']) ? true : $access);
			}
		}
		return $access;
	}

	/**
	* check the visibility access by extend options,
	* by action, custom action, board and language
	*/
	function _checkExtOpts($show)
	{
		global $context;

		// any element exist?
		foreach($this->cfg['config']['ext_opts'] as $item)
			$show = (!empty($item) ? false : $show);

		if($show)
			return $show;

		// check all elements
		foreach($this->cfg['config']['ext_opts'] as $item => $value)
		{
			// convert elements for simpler checking
			$data = array();
			if(!empty($value))
			{
				if($item == 'pmxcust')
				{
					$tmp = preg_split('~\,~', $value, -1, PREG_SPLIT_NO_EMPTY);
					foreach($tmp as $d)
					{
						$d = trim($d);
						if($d{0} == '^')
							$data[substr($d, 1)] = 0;
						else
							$data[$d] = 1;
					}
				}
				else
				{
					foreach($value as $d)
					{
						$tmp = preg_split('~\=~', $d, -1, PREG_SPLIT_NO_EMPTY);
						if(isset($tmp[0]) && isset($tmp[1]))
							$data[$tmp[0]] = $tmp[1];
					}
				}

				// now check by item
				switch($item)
				{
					// actions...
					case 'pmxact':
					case 'pmxcust':
						// frontpage ?
						if(empty($_REQUEST['action']) && empty($context['current_board']) && empty($context['current_topic']))
							$show = (array_key_exists('frontpage', $data) ? (boolean) $data['frontpage'] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));

						// global on boards ?
						if(!empty($_REQUEST['board']))
							$show = (array_key_exists('boards', $data) ? (boolean) $data['boards'] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));

						// gobal on topics ?
						if(!empty($_REQUEST['topic']))
							$show = (array_key_exists('topics', $data) ? (boolean) $data['topics'] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));

						// Singlepage ?
						if($item == 'pmxcust' && !empty($_REQUEST['action']) && $_REQUEST['action']{0} == '@')
						{
							// special handling custom page actions
							if(!empty($value) && strpos($value, '@') !== false)
							{
								foreach($data as $pgn => $v)
								{
									if(substr($pgn, -1) == '.')
										$show = (substr($_REQUEST['action'], 0, strlen($pgn)-1) == substr($pgn, 0, -1) ? (boolean) $v : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
									else
										$show = ($_REQUEST['action'] == $pgn ? (boolean) $v : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
								}
							}
							else
								$show = (array_key_exists('pages', $data) ? (boolean) $data['pages'] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
						}

						// any other action ?
						elseif(isset($_REQUEST['action']))
						{
							if($_REQUEST['action']{0} == '@')
								$show = (array_key_exists('pages', $data) ? (boolean) $data['pages'] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
							else
								$show = (array_key_exists($_REQUEST['action'], $data) ? (boolean) $data[$_REQUEST['action']] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
						}
					break;

					// specific board ?
					case 'pmxbrd':
						$show = (!empty($context['current_board']) && array_key_exists($context['current_board'], $data) ? (boolean) $data[$context['current_board']] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
					break;

					// language ?
					case 'pmxlng':
						$show = (isset($context['user']['language']) && array_key_exists($context['user']['language'], $data) ? (boolean) $data[$context['user']['language']] : (array_key_exists('any', $data) ? (boolean) $data['any'] : $show));
					break;
				}
			}
		}
		return $show;
	}
}

/**
* @class PortaMxC_SystemBlock
* The Global Systemblock Class.
* @see PortaMx_BlocksClass.php
* @author Copyright by PortaMx - http://portamx.com
*/
 class PortaMxC_SystemBlock extends PortaMxC_Blocks
{
	/**
	* The display block Methode.
	* ShowBlock prepare the frame, header and the body of each block.
	* Load the language and css if available.
	* After frame, header and body is prepared, the block depended content output is called.
	*/
	function pmxc_ShowBlock($count)
	{
		global $context, $user_info, $settings, $options, $scripturl, $txt;

		// get the block title for user language have it one or english for default
		$blocktitle = (isset($this->cfg['config']['title'][$user_info['language']]) ? $this->cfg['config']['title'][$user_info['language']] : $this->cfg['config']['title']['english']);

		echo '
					<div id="pmx_block'. $this->cfg['id'] .'" ';
//					<div ';

		// have a Frame, show it
		if(!empty($this->cfg['config']['visuals']['frame']))
			echo 'class="'. $this->cfg['config']['visuals']['frame'] .'" style="margin-bottom:'. (empty($count) ? '1' : $context['PortaMx']['settings']['panels']['padding']) .'px; overflow:hidden;"';
		else
			echo 'style="margin-bottom:'. (empty($count) ? '1' : $context['PortaMx']['settings']['panels']['padding']) .'px; overflow:hidden;"';
		echo '>';

		// have a header, show it
		if((!empty($this->cfg['config']['visuals']['header']) && $this->cfg['config']['visuals']['header'] != 'none') || (empty($this->cfg['config']['visuals']['header']) && !empty($this->cfg['config']['visuals']['body'])))
		{
			echo '
						<div id="pmx_block_tbar'. $this->cfg['id'] .'" class="'. (!empty($this->cfg['config']['visuals']['header']) ? $this->cfg['config']['visuals']['header'] : $this->cfg['config']['visuals']['body']) .'" style="min-height:16px;padding:2px 4px;">';

			// the title align
			if(!isset($this->cfg['config']['title_align']))
				$this->cfg['config']['title_align'] = 'left';
			switch($this->cfg['config']['title_align'])
			{
				case 'left':
					$imgalign = empty($context['right_to_left']) ? 'right' : 'left';
					$txtalign = empty($context['right_to_left']) ? 'left' : 'right';
					break;
				case 'right':
					$imgalign = empty($context['right_to_left']) ? 'left' : 'right';
					$txtalign = empty($context['right_to_left']) ? 'right' : 'left';
					break;
				case 'center':
					$imgalign = empty($context['right_to_left']) ? 'right' : 'left';
					$txtalign = 'center';
			}

			$blocktitle = htmlspecialchars($blocktitle, ENT_QUOTES);
			$jstitle = htmlspecialchars(addslashes($blocktitle), ENT_QUOTES);

			// show the collapse, if set and have a header
			if($this->cfg['config']['collapse'] == 1)
				echo '
							<span onclick="'. $this->cfg['side'] .'block'. $this->cfg['id'] .'.toggle(); setBlockUpshrinkTitle(\'upshrinkImg'. $this->cfg['id'] .'\', \''. $jstitle . $txt['pmx_ec_block'] .'\');" style="cursor:pointer;">
								<img id="upshrinkImg'. $this->cfg['id'] .'" align="'. $imgalign .'" src="'. $context['PMx_imageurl'] .(empty($options['collapse'. $this->cfg['id']]) ? 'collapse' : 'expand'). '.gif" alt="*" title="'. (empty($options['collapse'. $this->cfg['id']]) ? $txt['pmx_collapse'] : $txt['pmx_expand']) . $blocktitle . $txt['pmx_ec_block'] .'" />
							</span>';

			// show the title icon is set
			if(!empty($this->cfg['config']['title_icon']))
				echo '
							<span>
								<img align="'. ($imgalign == 'left' ? 'right' : 'left') .'" style="padding-'. $imgalign .':4px;" src="'. $context['PMx_Iconsurl'] . $this->cfg['config']['title_icon'] .'" alt="" title="'. $blocktitle . '" />
							</span>';

			// if quickedit link the title to blockedit? 
			if(!empty($context['PortaMx']['quickedit']) && $this->_modAccess(true))
				echo '<div style="text-align:'. $txtalign .';">
								<a href="'. $scripturl .'?action=admin;area=pmx_blocks;sa='. $this->cfg['side'] .';blockedit='. $this->cfg['id'] .';'. $context['session_var'] .'=' .$context['session_id'] .'">
									'. $blocktitle .'
								</a>';
			// else show the title normal
			else
				echo '
							<div style="text-align:'. $txtalign .'">'. $blocktitle;

			echo '
							</div>
						</div>';
		}

		// show content frame
		echo ' 
						<div id="upshrinkBlock'. $this->cfg['id'] .'"' .(!empty($this->cfg['config']['visuals']['body']) ? ' class="'. $this->cfg['config']['visuals']['body'] .'"' : ''). ' style="padding:'. (isset($this->cfg['config']['innerpad']) ? $this->cfg['config']['innerpad'] : '4').'px;' .(empty($options['collapse'. $this->cfg['id']]) ? '' : ' display:none;'). '">
							<div';

		// have a bodytext class ?
		if(!empty($this->cfg['config']['visuals']['bodytext']))
			echo ' class="'. $this->cfg['config']['visuals']['bodytext'] .'"';

		// have overflow, maxheight?
		if(!empty($this->cfg['config']['overflow']))
			echo ' style="'. (isset($this->cfg['config']['maxheight']) && !empty($this->cfg['config']['maxheight']) ? 'max-height:'. $this->cfg['config']['maxheight'] .'px; ' : '') .'overflow:'. $this->cfg['config']['overflow'] .';"';

		echo '>';

		/*
		* Because each block can have a own content output,
		* whe have now to call the block depended methode.
		*/
		$this->pmxc_ShowContent();

		echo '
							</div>
						</div>
					</div>';
	}

	/**
	* getCacheStatus returns the current cache status.
	* If the cache disabled, it returns 0.
	* if a refresh necessary it return a value less then 0
	*/
	function pmxc_getCacheStatus()
	{
		$result = 0;
		// check cache ..
		if(!empty($this->cfg['cache']))
		{
			$result = ($this->cfg['cacherefresh'] - time());
			if(empty($result))
				$result = -1;
		}
		return $result;
	}

	/**
	* updateCache save the next refresh time to the database.
	*/
	function pmxc_updateCache($ctime = 0, $UpdCache = false)
	{
		global $smcFunc;

		if(empty($ctime))
			$ctime = $this->cfg['cache'];
		else
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}portamx_blocks
				SET cacherefresh = {int:cacherefresh},
				'. ($UpdCache ? 'cache = {int:cache},' : '') .'
				content = {string:content}
				WHERE id = {int:id}',
			array(
				'id' => $this->cfg['id'],
				'cacherefresh' => time() + ($ctime * 60),
				'cache' => $ctime,
				'content' => $this->cfg['content'],
			)
		);
	}

	/**
	* InitContent returns the visibility flag.
	* This is mostly overwrite.
	*/
	function pmxc_InitContent()
	{
		// return the visibility flag (true/false)
		return $this->visible;
	}

	/**
	* ShowContent outputs the content of a block.
	* This is often overwrite.
	*/
	function pmxc_ShowContent()
	{
		// Write out the content
		echo $this->cfg['content'];
	}
}
?>