<?php
/** 
* @file LoadData.php
* LoadData holds all subroutines for Portamx. 
*
* @author PortaMx - Portal Management Extension
* @author Copyright 2008-2009 by PortaMx - http://portamx.com
* @Version: 0.957
* @Date: 11.03.2009
*/

if(!defined('PortaMx'))
	die('This file can\'t be run without PortaMx');

/**
* @class XmlElement
* XML elements for the RSS reader
*/
class XmlElement 
{
	var $name;
	var $attributes;
	var $content;
	var $childs;
};

/**
* Get all data from a RSS feed url
* Returns a header array and a post array
*/
function getRSSfeedPosts(&$feedheader, $feedurl, $maxentrys, $resposetime)
{
	$feedpost = array();

	// get the xml file from url
	$XmlRoot = ParseXmlurl($feedurl, $resposetime);
	if(!empty($XmlRoot)) 
	{
		$feedtyp = strtolower($XmlRoot->name);

		if($feedtyp == 'feed')
		{
			$Alnk = GetFirstChildByName($XmlRoot, 'link');
      if($Alnk)
				$hLink = GetAttribByName($Alnk, 'href');
			else
				$hLink = '';

			$feedheader = array(
				'title' => GetFirstChildContentByName($XmlRoot, 'title'),
				'link' => $hLink,
				'desc' => GetFirstChildContentByName($XmlRoot, 'tagline'),
			);

			// find / replace for atom date string
			$dtfnd = array('T', 'Z');
			$dtrep = array(' ', ' ');
		}
		else
		{
			$ttl = GetFirstChildContentByPath($XmlRoot, 'channel/ttl');
			if(GetFirstChildContentByPath($XmlRoot, 'channel/sy:updateperiod'))
			{
				$period = GetFirstChildContentByPath($XmlRoot, 'channel/sy:updateperiod');
				$freq = GetFirstChildContentByPath($XmlRoot, 'channel/sy:updatefrequency');
				if($period == 'hourly')
					$ttl = $freq * 60;
				if($period == 'daily')
					$ttl = $freq * (24*60);
			}
			$feedheader = array(
				'title' => GetFirstChildContentByPath($XmlRoot, 'channel/title'),
				'link' => GetFirstChildContentByPath($XmlRoot, 'channel/link'),
				'desc' => GetFirstChildContentByPath($XmlRoot, 'channel/description'),
				'ttl' => $ttl,
			);
		}

		if($feedtyp == 'rss')
			$elmlist = GetChildByPathAndName($XmlRoot, 'channel', 'item');
		else
			$elmlist = $XmlRoot->childs;

		if(!empty($elmlist))
		{
			// get all the items
			foreach ($elmlist as $elm) 
			{
				// is a RSS or RDF feed ?
				if($feedtyp == 'rss' || $feedtyp == 'rdf:rdf') 
				{
					if($feedtyp == 'rdf:rdf' && strtolower($elm->name) != 'item')
						continue;

					$poster = GetFirstChildContentByName($elm, 'author');
					if(empty($poster))
						$poster = GetFirstChildContentByName($elm, 'dc:creator');

					$date = GetFirstChildContentByName($elm, 'pubDate');
					if(empty($date)) 
						$date = GetFirstChildContentByName($elm, 'dc:date');
					if(!empty($date))
						$date = htmlspecialchars(strip_tags(timeformat(strtotime($date))));

					$category = GetFirstChildContentByName($elm, 'category');
					if(empty($category))
						$category = GetFirstChildContentByName($elm, 'dc:category');

					$subject = GetFirstChildContentByName($elm, 'subject');
					if(empty($subject))
						$subject = GetFirstChildContentByName($elm, 'title');

					$feedpost[] = array(
						'subject' => $subject,
						'slink' => GetFirstChildContentByName($elm, 'link'),
						'tlink' => '',
						'poster' => $poster,
						'plink' => '',
						'date' => $date,
						'category' => $category,
						'board' => '',
						'blink' => '',
						'message' => GetFirstChildContentByName($elm, 'description'),
						'contenc' => GetFirstChildContentByName($elm, 'content:encoded'),
					);
				}
				// or is a SMF feed ?
				elseif($feedtyp == 'smf:xml-feed')
				{
					$feedpost[] = array(
						'subject' => GetFirstChildContentByName($elm, 'subject'),
						'slink' => GetFirstChildContentByName($elm, 'link'),
						'tlink' => str_replace('.new#new', '.0', GetFirstChildContentByPath($elm, 'topic/link')),
						'poster' => GetFirstChildContentByPath($elm, 'poster/name'),
						'plink' => GetFirstChildContentByPath($elm, 'poster/link'),
						'date' => GetFirstChildContentByName($elm, 'time'),
						'category' => '',
						'board' => GetFirstChildContentByPath($elm, 'board/name'),
						'blink' => GetFirstChildContentByPath($elm, 'board/link'),
						'message' => GetFirstChildContentByName($elm, 'body'),
						'contenc' => '',
					);
				}
				// or is a Atom feed ?
				elseif($feedtyp == 'feed')
				{
					if(strtolower($elm->name) != 'entry')
						continue;

					$date = str_replace($dtfnd, $dtrep, GetFirstChildContentByName($elm, 'published'));
					if(!empty($date))
						$date = htmlspecialchars(strip_tags(timeformat(strtotime($date))));

					$sLink = GetFirstChildContentByName($elm, 'id');
					$tLink = '';
					if(strpos($sLink, '.msg') !== false)
						$tLink = substr($sLink, 0, strpos($sLink, '.msg')) .'.0';

					$feedpost[] = array(
						'subject' => GetFirstChildContentByName($elm, 'title'),
						'slink' => GetFirstChildContentByName($elm, 'id'),
						'tlink' => $tLink,
						'poster' => GetFirstChildContentByPath($elm, 'author/name'),
						'plink' => GetFirstChildContentByPath($elm, 'author/uri'),
						'date' => $date,
						'category' => GetFirstChildContentByPath($elm, 'category/label'),
						'board' => '',
						'blink' => '',
						'message' => GetFirstChildContentByName($elm, 'summary'),
						'contenc' => '',
					);
				}

				if(!empty($maxentrys))
				{
					$maxentrys--;
					if($maxentrys <= 0)
						break;
				}
			}
		}
	}
	return $feedpost;
}

/**
* Parse a xml stream from a url
* Returns the xml content
*/
function ParseXmlurl($feedurl, $resposetime, $headerstart = '<?xml') 
{
	global $context, $txt;

	// get host and domain from feedurl
	preg_match('@^(?:http://)?([^/|^?]+)@i', $feedurl, $host);
	if(!isset($host[1]))
		return '';

	preg_match('@'. $host[1] .'(.*)@i', $feedurl, $domain);
	if(!isset($domain[1]))
		return '';

	// prepare the http header
	$header = "GET ". $domain[1] ." HTTP/1.1\r\n";
	$header .= "Host: ". $host[1] ."\r\n";  
	$header .= "Connection: Close\r\n";  
	$header .= "\r\n";   

	// open the socket
	$handle = @fsockopen($host[1], 80, $eNbr, $eStr, intval($resposetime));
  if(!$handle)
	{
		log_error(sprintf($txt['feed_response_error'], $feedurl, $resposetime));
		return '';
	}

	$xml = '';
	fputs($handle, $header);
	while(!feof($handle))
		$xml .= fgets($handle, 8092);

	fclose($handle);

  // remove http header from result
  if(strpos($xml, $headerstart) !== false)
    $xml = substr($xml, strpos($xml, $headerstart));
  else
    return '';
  if($headerstart == '<?xml')
	  return ParseXml($xml);
  else
    return $xml;
}

/**
* Parse a xml stream
* Returns the xml content
*/
function ParseXml($xml) 
{
	global $context, $txt;

	$parser = xml_parser_create($context['character_set']);
	xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	xml_parse_into_struct($parser, $xml, $tags);
	xml_parser_free($parser);

	$elements = array();
	$stack = array();
	foreach ($tags as $tag) 
	{
		$index = count($elements);
		if(($tag['type'] == "complete") || ($tag['type'] == "open")) 
		{
			$elements[$index] = new XmlElement;
			if(isset($tag['tag'])) 
				$elements[$index]->name = $tag['tag'];
			if(isset($tag['attributes'])) 
				$elements[$index]->attributes = $tag['attributes'];
			if(isset($tag['value'])) 
				$elements[$index]->content = $tag['value'];
			if($tag['type'] == "open") 
			{
				$elements[$index]->childs = array();
				$stack[count($stack)] = &$elements;
				$elements = &$elements[$index]->childs;
			}
		}
		if($tag['type'] == "close") 
		{
			$elements = &$stack[count($stack) - 1];
			unset($stack[count($stack) - 1]);
		}
	}
	return isset($elements[0]) ? $elements[0] : array();
}

/**
* Get Child By Path And Name
* Returns the child
*/
function GetChildByPathAndName($XmlRoot, $sPath, $sName) 
{
	$aPath = split('/', $sPath);
	$oRes = array();
	$elm = $XmlRoot;
	if(!empty($sPath))
	{
		foreach ($aPath as $p) 
		{
			$elm = GetFirstChildByName($elm, $p);
			if(empty($elm)) 
				return '';
		}
	}
	foreach ($elm->childs as $c) 
	{
		if (strcasecmp($c->name, $sName) == 0) 
			$oRes[count($oRes)] = $c;
	}
	return $oRes;
}

/**
* Get First ChildContent By Path
* Returns the content
*/
function GetFirstChildContentByPath($XmlRoot, $sPath) 
{
	$elm = GetFirstChildByPath($XmlRoot, $sPath);
	if(!empty($elm)) 
		return $elm->content; 
	else 
		return '';
}

/**
* Get First ChildContent By Name
* Returns the content
*/
function GetFirstChildContentByName($oParent, $sName) 
{
	$elm = GetFirstChildByName($oParent, $sName);
	if(!empty($elm)) 
		return $elm->content; 
	else 
		return '';
}

/**
* Get First Child By Path
* Returns the name
*/
function GetFirstChildByPath($XmlRoot, $sPath, $bCase = false) 
{
	$aPath = split('/', $sPath);
	$elm = $XmlRoot;
	foreach ($aPath as $p)
	{
		$elm = GetFirstChildByName($elm, $p);
		if(empty($elm)) 
			return '';
	}
	return $elm;
}

/**
* Get First Child By Name
* Returns the name
*/
function GetFirstChildByName($oParent, $sName, $bCase = false) 
{
	if(count($oParent->childs) > 0) 
	{
		foreach ($oParent->childs as $c)
			if(strcasecmp($c->name, $sName) == 0)
				return $c;
	}
	return '';
}

/**
* Get Attribute By Name
* Returns the name
*/
function GetAttribByName($XmlNode, $sName) 
{
	$aAttributes = array_change_key_case($XmlNode->attributes, CASE_LOWER);
  if(isset($aAttributes[$sName])) 
		return $aAttributes[$sName];
	else 
		return '';
}

/**
* hexstring to charstring
*/
function unhex($h)
{
	$s = '';
	for($i = 0; $i < strlen($h); $i += 2)
		$s .= chr(hexdec($h{$i} . $h{$i+1}));
	return $s;
}

/**
* check if a download from a Download-Block.
*/
function checkDownloadACS($fld)
{
	global $context, $user_info, $smcFunc;

	// get group access
  $acsgrp = isset($_SESSION['pxm_download_acs'][$fld]) ? $_SESSION['pxm_download_acs'][$fld] : array();

	// check if enabled for the usergroup
	$show = AllowedTo('admin_forum') ? true : false;
	if(is_array($acsgrp))
	{
		foreach($user_info['groups'] as $g)
			$show = (in_array($g, $acsgrp) ? true : $show);
	}
	if(!$show)
		isAllowedTo('view_attachments');

	// notice cacherefresh for the download blocks
	$_SESSION['pxmcontent_refresh']['download'] = true;

	// get the filedata from attach table
	$request = $smcFunc['db_query']('', '
			SELECT a.id_folder, a.filename, a.fileext, a.id_attach, a.attachment_type, a.mime_type, a.approved
			FROM {db_prefix}attachments AS a
				 INNER JOIN {db_prefix}messages AS m ON (m.id_msg = a.id_msg)
				 INNER JOIN {db_prefix}boards AS b ON (b.id_board = m.id_board)
			WHERE a.id_attach = {int:attach}
			LIMIT 1',
			array(
				 'attach' => $_REQUEST['attach'],
			)
	);
	return $request;
}
 
/**
* read the settings from database.
*/
function PortaMx_getSettings()
{
	global $smcFunc, $context, $boarddir, $options, $forum_version;
 
	$request = $smcFunc['db_query']('', '
			SELECT id, config 
			FROM {db_prefix}portamx_settings
			ORDER BY id
			LIMIT 4',
		array()
	);

	$nrd = $smcFunc['db_num_rows']($request);
  $keys = array(1 => '', 2 => 'keydata', 3 => 'dbreads', 4 => 'server');
	if($nrd > 0)
	{
		while($row = $smcFunc['db_fetch_assoc']($request))
		{
			if(empty($keys[$row['id']]))
				$result = unserialize($row['config']);
			elseif($keys[$row['id']] == 'dbreads')
				$result[$keys[$row['id']]] = sha1($result['keydata']['key']) == $row['config'];
			elseif($keys[$row['id']] == 'server')
				$context['PortaMx'][$keys[$row['id']]] = unserialize($row['config']);
			else
				$result[$keys[$row['id']]] = unserialize($row['config']);
		}
		$smcFunc['db_free_result']($request);
	}
	if($nrd != 4 || empty($result['dbreads']))
		fatal_error('portamx_setting: Invalid database value reached.');

	// get the usergroup
	$request = $smcFunc['db_query']('', '
			SELECT id_group
			FROM {db_prefix}members
			WHERE id_member = {int:idmem}
			LIMIT 1',
		array('idmem' => $context['user']['id'])
	);

	$row['id_group'] = 0;
	if($smcFunc['db_num_rows']($request) > 0)
	{
		$row = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);
	}
	$result['usergroup'] = $row['id_group'];

	return $result;
}

/**
* check the show/hide panel, empty panels are not show.
*/
function getPanelsToShow($action, $PageReq = false)
{
	global $smcFunc, $context, $user_info, $maintenance;

	// get active panels for this action
	$activePanels = array('none');
	$allsides = $context['PortaMx']['block_sides'];
	if($context['PortaMx']['settings']['frontpage'] == 'none')
		$allsides = array_diff($allsides, array('front'));

	foreach($allsides as $side)
	{
		$hidepanels = isset($context['PortaMx']['settings'][$side .'_panel']['hide']) ? $context['PortaMx']['settings'][$side .'_panel']['hide'] : array();
		$customhide = isset($context['PortaMx']['settings'][$side .'_panel']['custom_hide']) ? preg_split('~\,~', $context['PortaMx']['settings'][$side .'_panel']['custom_hide'], -1, PREG_SPLIT_NO_EMPTY) : array();
		$hidepanels = array_merge($hidepanels, $customhide);

		// Page request?
		if($side == 'pages' && !$PageReq)
			$context['PortaMx']['show_'. $side .'panel'] = false;

		// wap?
		if($action == 'wap2')
			$context['PortaMx']['show_'. $side .'panel'] = false;
		elseif(!empty($hidepanels))
		{
			if(in_array($action, $hidepanels))
				$context['PortaMx']['show_'. $side .'panel'] = false;
			elseif(!empty($customhide) && strpos($context['PortaMx']['settings'][$side .'_panel']['custom_hide'], '@') !== false)
			{
				$show = true;
				foreach($customhide as $pgn)
				{
					if($pgn{0} == '@' && isset($_REQUEST['action']))
					{
						if(substr($pgn, -1) == '.')
							$show = (substr($_REQUEST['action'], 0, strlen($pgn)-1) == substr($pgn, 0, -1) ? false : $show);
						else
							$show = ($_REQUEST['action'] == $pgn ? false : $show);
					}
				}
				$context['PortaMx']['show_'. $side .'panel'] = $show;
				if($show)
					$activePanels[] = $side;
			}
			else
				$activePanels[] = $side;
		}
		else
			$activePanels[] = $side;
	}

	if(isset($_SESSION['pxm_paneloff']))
	{
		$activePanels = array_diff($activePanels,	$_SESSION['pxm_paneloff']);
		foreach($_SESSION['pxm_paneloff'] as $side)
			$context['PortaMx']['show_'. $side .'panel'] = false;
	}

	// read the panels from database
	$result = array();
	$request = $smcFunc['db_query']('', '
			SELECT * FROM {db_prefix}portamx_blocks
			WHERE active = 1 AND side IN ({array_string:sides})
			ORDER BY side ASC, pos ASC',
		array('sides' => $activePanels)
	);

	if($smcFunc['db_num_rows']($request) > 0)
	{
		while($row = $smcFunc['db_fetch_assoc']($request))
		{
			$blocktype = 'pmxc_'. $row['blocktype'];
			// load the class file if not loaded
			if(!class_exists($blocktype)) 
				require_once($context['PMx_classdir'] . $row['blocktype'] .'.php');

			$objHdl = new $blocktype($row);
			if($objHdl->pmxc_InitContent())
				$result[$row['side']][] = $objHdl;
			unset($objHdl);
		}
		$smcFunc['db_free_result']($request);
	}

	// switch off empty panels
	foreach($activePanels as $side)
	{
		// Hide all panels on maintenance for normal user
		if(!empty($maintenance) && empty($user_info['is_admin']))
			$context['PortaMx']['show_'. $side .'panel'] = false;
		else
		{
			$context['PortaMx']['show_'. $side .'panel'] = (isset($result[$side]) && !empty($result[$side]));
			if($side == 'pages' && $PageReq && empty($result[$side]))
				$context['PortaMx']['show_'. $side .'panel'] = true;

			if(empty($context['PortaMx']['show_'. $side .'panel']))
				$context['PortaMx']['collapse'][$side] =  0;
		}
	}

	return $result;
}

/**
* get all smf user groups.
*/
function PortaMx_getUserGroups()
{
	global $smcFunc, $context, $txt;

	// guest & normal members
	$result = array(
		0 => array(
			'id' => '-1',
			'name' => $txt['pmx_guest'],
		),
		1 => array(
			'id' => '0',
			'name' => $txt['pmx_ungroupedmembers'],
		),
	);

	// get SMF membergroups
	$request = $smcFunc['db_query']('', '
			SELECT id_group, group_name
			FROM {db_prefix}membergroups
			WHERE min_posts = -1 AND id_group > 1
			ORDER BY id_group',
		array()
	);
	while($row = $smcFunc['db_fetch_assoc']($request))
	{
		$result[] = array(
			'id' => $row['id_group'],
			'name' => $row['group_name'],
		);
	}
	$smcFunc['db_free_result']($request);
	return $result;
}

/**
* get all smf and PortaMxlanguages.
*/
function PortaMx_getLanguages()
{
	global $context, $settings;

	// check SMF languages
	$lang_dir = $settings['default_theme_dir'] . '/languages';
	$dir = dir($lang_dir);
	while ($entry = $dir->read())
	{
		preg_match('~^index\.?([a-zA-Z0-9_-]+)\.php~', $entry, $match);
		if(!empty($match))
			$resultSMF[$match[1]] = $match[1];
	}
	$dir->close();

	// check PortaMx languages
	$lang_dir = $settings['default_theme_dir'] . '/languages/PortaMx';
	$dir = dir($lang_dir);
	while ($entry = $dir->read())
	{
		preg_match('~^PortaMx\.?([a-zA-Z0-9_-]+)\.php~', $entry, $match);
		if(!empty($match))
			$resultPortaMx[$match[1]] = $match[1];
	}
	$dir->close();

	$result = array_intersect($resultSMF, $resultPortaMx);
	if(empty($result))
		return 'english';
	else
		return $result;
}

/**
* get all smf themes.
* idonly = false: Id's and names.
* idonly = true: only Id's.
*/
function PortaMx_getsmfThemes($idonly = false)
{
	global $context, $smcFunc;

	$result = null;

	// get the theme defaults
	$request = $smcFunc['db_query']('', '
			SELECT variable, value
			FROM {db_prefix}settings
			WHERE variable IN({array_string:var})',
		array('var' => array('theme_default', 'knownThemes'))
	);
	while($row = $smcFunc['db_fetch_assoc']($request))
		$themedef[$row['variable']] = $row['value'];
	$smcFunc['db_free_result']($request);

	$smfenabled = preg_split('~\,~', $themedef['knownThemes'], -1, PREG_SPLIT_NO_EMPTY);
	if($idonly)
		return $smfenabled;

	// get all themes
	$request = $smcFunc['db_query']('', '
			SELECT m.id_theme as mem_theme, t.id_theme, t.variable, t.value
			FROM {db_prefix}themes as t
			LEFT JOIN {db_prefix}members as m on({int:mem} = m.id_member)
			WHERE variable in({array_string:val})
			ORDER BY value, id_theme',
		array(
			'val' => array('name', 'images_url'),
			'mem' => $context['user']['id']
		)
	);
	while($row = $smcFunc['db_fetch_assoc']($request))
	{
		if(is_null($row['mem_theme']))
			$row['mem_theme'] = $themedef['theme_default'];
		$result[$row['id_theme']][$row['variable']] = $row['value'];
		if($row['variable'] == 'name')
		{
			$result[$row['id_theme']]['isdefault'] = ($row['id_theme'] == $themedef['theme_default']);
			$result[$row['id_theme']]['usertheme'] = ($row['mem_theme'] == $row['id_theme']);
			$result[$row['id_theme']]['smfenabled'] = (in_array($row['id_theme'], $smfenabled));
		}
	}
	$smcFunc['db_free_result']($request);
	
	return $result;
}

/**
* get all smf boards.
*/
function PortaMx_getsmfBoards()
{
	global $context, $modSettings, $smcFunc;

	$result = null;
	$request = $smcFunc['db_query']('', '
			SELECT id_board, name
			FROM {db_prefix}boards
			WHERE id_board != {int:excl}
			ORDER BY board_order',
		array(
			'excl' => isset($modSettings['recycle_board']) ? $modSettings['recycle_board'] : '0'
		)
	);
	while($row = $smcFunc['db_fetch_assoc']($request))
		$result[] = array(
			'id' => $row['id_board'],
			'name' => $row['name']
		);

	$smcFunc['db_free_result']($request);
	return $result;
}

/**
* get available block css files.
*/
function PortaMx_getAllCssFiles()
{
	global $context;

	$result = array('');
	$dir = dir($context['PMx_blockscssdir']);
	while ($entry = $dir->read())
	{
		preg_match('~([a-zA-Z0-9_-]+)\.css~', $entry, $match);
		if(!empty($match))
			$result[] = $match[1];
	}
	$dir->close();
	return $result;
}

/**
* load all not loaded Classfiles.
*/
function PortaMx_loadClassfiles($path)
{
	$dir = dir($path);
	while ($entry = $dir->read())
	{
		preg_match('~([a-zA-Z0-9_-]+)_adm.php~', $entry, $match);
		if(!empty($match))
		{
			// load the class file if not loaded
			if(!class_exists('pmxc_'. $match[1])) 
				require_once($path . $match[0]);
		}
	}
	$dir->close();
}

/**
* show the blocks for a side.
*/
function PortaMx_ShowBlocks($side, $spacer = 0)
{
	global $context, $txt;

	if(isset($context['PortaMx']['viewblocks'][$side]))
	{
		$count = count($context['PortaMx']['viewblocks'][$side]) + $spacer;
		foreach($context['PortaMx']['viewblocks'][$side] as $ObjHdl)
		{
			$count--;
			$ObjHdl->pmxc_ShowBlock($count);
		}
	}
	else
	{
		if($side == 'pages')
		{
			echo '
			<div style="width:80%; margin:0 auto; text-align:center;">
				<div class="titlebg" style="padding:5px 0px;">'. $txt['page_reqerror_title'] .'</div>
				<div class="tborder">
					<div class="smallpadding windowbg middletext">
						'. $txt['page_reqerror_msg'] .'<br /><br />
						<input type="button" name="back" value="'. $txt['page_reqerror_button'] .'" onclick="window.history.back()" />
					</div>
				</div>
			</div>';
		}
	}
}

/**
* write out a side block (left or right).
*/
function Show_Block($side)
{
	global $context, $options, $txt;

	if(empty($context['PortaMx']['settings'][strtolower($side).'_panel']['collapse']) && $context['PortaMx']['xbar_'.strtolower($side)])
		echo '
				<div id="xbar'.strtolower($side).'" title="'. (empty($options['collapse_'.strtolower($side)]) ? $txt['pmx_hidepanel'] : $txt['pmx_showpanel']) . $txt['pmx_block_panels'][strtolower($side)] .'" onclick="'.$side.'_Panel()"></div>';
	echo '
				<div id="upshrink'.$side.'Bar" style="width:'. $context['PortaMx']['settings'][strtolower($side).'_panel']['size'] .'px; margin-'.($side == 'Left' ? 'right' : 'left').':'. $context['PortaMx']['settings']['panels']['padding'] .'px;';

	// overflow action set?
	if(!empty($context['PortaMx']['settings']['panels']['overflow']))
		echo ' overflow:'. $context['PortaMx']['settings']['panels']['overflow'] .';';

	if(empty($context['PortaMx']['settings'][strtolower($side).'_panel']['collapse']) && !empty($options['collapse_'.strtolower($side)]))
		echo ' display:none;';
	echo '">';

	PortaMx_ShowBlocks(strtolower($side));

	echo '
				</div>';
}

/**
* remove html, the $ and array[element].
*/
function PortaMx_makeSafe($value)
{
  return preg_replace('~(\[\/?)(.+)([^\]]*\])|\$~', '', preg_replace('~(<\/?)(.+)([^>]*>)~', '', $value));
}

/**
* remove all unnecessary <br> and lf/cr from content.
*/
function PortaMx_makeSafeContent($value, $type = '')
{
	global $context;

	// remove <br> and lf/cr from end
	$value = rtrim($value);
	if($type == 'html')
	{
		$res = true;
		while($res) 
		{
			$last = substr($value, -6);
			$res = preg_match('~([^<]+)?br[^>]*>~i', $last, $found);
			if(!empty($res))
				$value = rtrim(substr($value, 0, -(strlen($found[0])+1)));
		}
	}
	return $value;
}

/**
* remove Links from content.
* unlinkimg = false: images not unlink.
* unlinkimg = true: unlink images.
*/
function PortaMx_revoveLinks($content, $unlinkimg = false)
{
  global $modSettings, $txt;

	$found = preg_match_all("!<(a)\s[^>]*>(.+?)</\\1>!iS", $content, $matches);
	for($i = 0; $i < $found; $i++)
	{
		if(strpos($matches[2][$i], '<img') === false)
			$content = str_replace($matches[0][$i], $matches[2][$i], $content);
		elseif($unlinkimg)
			$content = str_replace($matches[0][$i], $matches[2][$i], $content);
	}

	// remove inside images
	if($unlinkimg)
	{
		$found = preg_match_all('!<img[^>]*>!iS', $content, $matches);
		if(!empty($found))
		{
			foreach($matches[0] as $data)
			{
				if(strpos($data, $modSettings['smileys_url']) === false)
					$content = str_replace($data, '', $content);
			}
		}
	}

	return $content;
}

/**
* Post teaser (shorten posts by given wordcount).
* remove_atags = true: remove Links from content.
* remove_atags = false: not remove Links from content.
*/
function PortaMx_Tease_posts($content, $wordcount, $remove_atags = false)
{
	global $context, $settings, $boarddir, $boardurl;

	// Define post teaser tags
	$blocks = 'p|li|dt|dd|address|form|pre|td|div|span|code|blockquote|pmxteaser';
	$teasemark = array('<pmxteaser>', '</pmxteaser>');
	$matches = null;
	$matches2 = null;
	$auto_close = array();

	if($remove_atags)
		$content = PortaMx_revoveLinks($content, $remove_atags);

	// set outside marker
	$content = $teasemark[0]. $content .$teasemark[1];

	$word_count = pmx_word_count($content);
	preg_match_all("!.*?<($blocks)[^>]*>.+?</\\1>!si", $content, $matches);
	$matches = $matches[0];
	$block_count = count($matches);

	$current_word_count = 0;
	$block_word_count = array();
	$done = false;
	for ($i = 0; $current_word_count < $wordcount && $i < $block_count && empty($done);)
	{
		$block_word_count[$i] = pmx_word_count($matches[$i]);
		$current_word_count += $block_word_count[$i];

		if ($current_word_count > $wordcount)
		{
			$current_word_count -= $block_word_count[$i];
			$block_word_count[$i] = 0;

			// calculate tags inside a block
			$f = preg_match_all('!<*[^>]*>!s', $matches[$i], $tmp);
			$tmp = $tmp[0];
			$matches[$i] = '';
			$t = 0;
			while($t < $f && empty($done))
			{
				$tcnt = pmx_word_count($tmp[$t]);
				if($current_word_count + $tcnt < $wordcount)
				{
					$current_word_count += $tcnt;
					$block_word_count[$i] += $tcnt;
					$matches[$i] .= $tmp[$t];
				}
				elseif($tmp[$t]{0} != '<')
				{
					while(pmx_word_count($tmp[$t]) + $block_word_count[$i] > $wordcount)
						$tmp[$t] = substr($tmp[$t], 0, strrpos($tmp[$t], ' '));
					$matches[$i] .= $tmp[$t] .' ...';
					$done = true;
				}
				$t++;
			}
		}
		$i++;
	}

	while($i > 0 && !isset($block_word_count[$i]))
		$i--;

	if ($current_word_count >= $wordcount && $i > 0)
	{
		$this_block_distance = $current_word_count - $wordcount;
		$last_block_distance = $wordcount - ($current_word_count - $block_word_count[$i]);
		if ($this_block_distance > $last_block_distance)
			$i--;
	}

	$new_content = $content;
	$tease = false;
	if($block_count == 0)
	{
		$tease = strpos($new_content, '<!--more-->');
		if(!$tease)
		{
			$words = pmx_word_count($new_content);
			if($words > $wordcount)
			{
				$tmp = $new_content;
				while(pmx_word_count($tmp) > $wordcount)
					$tmp = substr($tmp, 0, strrpos($tmp, ' '));
				$tease = strlen($tmp);
			}
		}
		if($tease)
		{
			$teasepos = strpos($new_content, '<div style="page-break-after: always;">');
			if($teasepos === false)
				$new_content = trim(substr($new_content, 0, $tease));
			else
				$new_content = trim(substr($new_content, 0, $teasepos));
				while(substr($new_content, -6) == '<br />')
					$new_content = substr($new_content, 0, strlen($new_content) - 6) .' ...';
		}
	}
	elseif($i <= $block_count)
	{
		for ($j = 0; $j <= $i;)
		{
			if(isset($matches[$j]))
			{
				$tease = strpos($matches[$j], '<!--more-->');
				if(!$tease)
				{
					$words = pmx_word_count($matches[$j]);
					if($words > $wordcount)
					{
						$tmp = $matches[$j];
						while(pmx_word_count($tmp) > $wordcount)
							$tmp = substr($tmp, 0, strrpos($tmp, ' '));
						$tease = strlen($tmp);
					}
				}
				if($tease)
				{
					$teasepos = strpos($new_content, '<div style="page-break-after: always;">');
					if($teasepos === false)
						$tmp = trim(substr($matches[$j], 0, $tease));
					else
						$tmp = trim(substr($matches[$j], 0, $teasepos));
					while(substr($tmp, -6) == '<br />')
						$tmp = substr($tmp, 0, strlen($tmp) - 6);
					$matches[$j] = $tmp .' ...';

					preg_match_all('!<(?:[a-zA-Z1-9]+)[^>]*>!i', $matches[$j], $matches2);
					$matches2 = $matches2[0];
					foreach ($matches2 as $id => $element)
					{
						if (preg_match('!^<([a-zA-Z1-9]+)[^>]*/>$!i', $element))
						{
							unset($matches2[$id]);
							continue;
						}
						$element = preg_replace('!^<([a-zA-Z1-9]+)[^>]*>$!i', "$1", $element);
						$auto_close[] = $element;
					}
					while($j <= $i)
					{
						$j++;
						if(isset($matches[$j]))
							unset($matches[$j]);
					}
				}
				else
				{
					preg_match_all('!<(?:[a-zA-Z1-9]+)[^>]*>!i', $matches[$j], $matches2);
					$matches2 = $matches2[0];
					foreach ($matches2 as $id => $element)
					{
						if (preg_match('!^<([a-zA-Z1-9]+)[^>]*/>$!i', $element))
						{
							unset($matches2[$id]);
							continue;
						}
						$element = preg_replace('!^<([a-zA-Z1-9]+)[^>]*>$!i', "$1", $element);
						$auto_close[] = $element;
					}
				}
			}
			$j++;
		}

		$new_content = '';
		for ($j = 0; $j <= $i; $j++)
		{
			if(isset($matches[$j]))
			{
				$temp = $matches[$j];
				foreach ($auto_close as $id => $element)
				{
					$pos = strpos(" " . $temp, "</$element>");
					if ($pos)
					{
						$temp = substr_replace($temp, '', $pos, strlen("</$element>"));
						unset($auto_close[$id]);
					}
				}
				$new_content .= $matches[$j];
			}
		}
		$auto_close = array_reverse($auto_close);
		foreach ($auto_close as $element)
			$new_content .= "</$element>";
	}

	$result = str_replace($teasemark, array('', ''), $new_content);
	$context['PortaMx']['is_teased'] = $tease;
	return $result;
}

/**
* get word cont for post_teaser.
*/
function pmx_word_count($text)
{
	return count(preg_split("/\s+/", strip_tags(str_replace(array('&#160;','&nbsp;'), array(' ', ' '), $text)), -1, PREG_SPLIT_NO_EMPTY));
}

/**
* change a theme permanent.
*/
function PortaMx_ChangeTheme($themeid, $redirurl)
{
	global $context, $smcFunc, $scripturl;

	$thid = PortaMx_makeSafeContent($themeid);
	if(in_array($thid, PortaMx_getsmfThemes(true)) && $context['user']['is_logged'])
	{
		$smcFunc['db_query']('', '
				UPDATE {db_prefix}members
				SET id_theme = {int:idtheme}
				WHERE id_member = {int:idmem}',
			array(
				'idtheme' => $thid,
				'idmem' => $context['user']['id']
			)
		);
	}
	PortaMx_ChangeRedirect($redirurl);
}

/**
* redirect to given url.
*/
function PortaMx_ChangeRedirect($redirurl)
{
	global $scripturl;

	// check the request and redirect
	redirectexit($scripturl . PortaMx_makeSafe(str_replace('|', ';', $redirurl)));
}

/**
* get the portamx template layer.
*/
function getTemplateLayer($layer = '')
{
	global $context;

	if($layer == 'portamx')
	{
		if(count($context['template_layers']) == 1)
			$context['template_layers'] = array();
		eval(unhex($context['PortaMx']['settings']['keydata']['key']));
	}
}

/**
* create the header for PortaMx.
*/
function PortaMx_headers($action = '')
{
	global $context, $settings, $txt, $options, $scripturl;

	if ($context['user']['is_guest'])
	{
		$options['collapse_left'] = !empty($_COOKIE['upshrinkLeft']);
		$options['collapse_right'] = !empty($_COOKIE['upshrinkRight']);
		$options['collapse_top'] = !empty($_COOKIE['upshrinkTop']);
		$options['collapse_bottom'] = !empty($_COOKIE['upshrinkBottom']);
  }

	// switch of xbarkeys on posting
	if($action == 'post')
		$context['PortaMx']['xbarkeys'] = 0;

	if(!empty($_REQUEST['board']))
		$tbpad = '0px';
	else
		switch ($action)
		{
			case '':
			case 'admin':
			case 'moderate':
			case 'profile':
				$tbpad = '0px';
			break;
			case 'forum':
				$tbpad = '5px';
			break;
			case 'search': 
			case 'search2':
			case 'calendar':
			case 'help':
			case 'unread':
				$tbpad = '7px';
			break;
			case 'pm':
			case 'mlist':
			case 'post':
			case 'stats':
			case 'who':
				$tbpad = '6px';
			break;
			default:
				$tbpad = '0px';
		}

	$context['html_headers'] .= '
	<style type="text/css">
	<!--
	.pmx_toppad { margin-top: '. (!empty($tbpad) ? '-'. $tbpad : '0') .'; }
	.pmx_barpad { padding-top: '. $tbpad .'; }
	-->
	</style>
	<!--[if lt IE 7]>
	<script defer type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/pngfix.js"></script>
	<![endif]-->
	<link rel="stylesheet" type="text/css" href="'. $settings['default_theme_url'] .'/PortaMx/portamx.css" />';

	if(empty($context['PortaMx']['settings']['disableHS']))
		$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'. $settings['default_theme_url'] .'/highslide/highslide.css" />';

	if(!empty($context['right_to_left']))
	{
		$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'. $settings['default_theme_url'] .'/PortaMx/portamx_rtl.css" />';

		if(empty($context['PortaMx']['settings']['disableHS']))
			$context['html_headers'] .= '
	<link rel="stylesheet" type="text/css" href="'. $settings['default_theme_url'] .'/highslide/highslide_rtl.css" />';
	}

	if(empty($context['PortaMx']['settings']['disableHS']))
		$context['html_headers'] .= '
	<script type="text/javascript" src="'. $settings['default_theme_url'] .'/highslide/highslide-full.packed.js"></script>
	<script language="JavaScript" type="text/javascript"><!-- // --><![CDATA[
		hs.graphicsDir = "'. $settings['default_theme_url'] .'/highslide/graphics/";
		hs.captionText = "'. $txt['pmx_hs_caption'] .'"; 
		hs.outlineType = "glossy-dark";
		hs.blockRightClick = true;
		hs.fadeInOut = true;
		hs.dimmingOpacity = 0.3;
		hs.showCredits = false;
		hs.expandSteps = 0;
		hs.restoreSteps = 0;
		hs.transitions = ["expand", "crossfade"];
		hs.enableKeyListener = false;
		hs.zIndexCounter = 10001;
		hs.registerOverlay({
			overlayId: "closebutton",
			position: "top right",
			fade: 2,
			useOnHtml: true
		});
	// ]]></script>';

	$context['pmx_toggleQuickEdit'] = isset($_COOKIE['pmx_toggleQuick']) ? $_COOKIE['pmx_toggleQuick'] : 'none';

	$context['html_headers'] .= '
	<script language="JavaScript" type="text/javascript"><!-- // --><![CDATA[
		var pmx_xBarKeys = '. (!empty($context['PortaMx']['xbarkeys']) ? 'true' : 'false') .';
		var xBarKeys_Status = pmx_xBarKeys;

		var leftPanel = new smfToggle("upshrinkLeft", '. (empty($options['collapse_left']) ? 'false' : 'true') .');
		leftPanel.useCookie('. ($context['user']['is_guest'] ? 1 : 0) .');
		leftPanel.setOptions("collapse_left", "'. $context['session_id'] .'");
		leftPanel.addTogglePanel("upshrinkLeftBar");

		var rightPanel = new smfToggle("upshrinkRight", '. (empty($options['collapse_right']) ? 'false' : 'true') .');
		rightPanel.useCookie('. ($context['user']['is_guest'] ? 1 : 0) .');
		rightPanel.setOptions("collapse_right", "'. $context['session_id'] .'");
		rightPanel.addTogglePanel("upshrinkRightBar");

		var topPanel = new smfToggle("upshrinkTop", '. (empty($options['collapse_top']) ? 'false' : 'true') .');
		topPanel.useCookie('. ($context['user']['is_guest'] ? 1 : 0) .');
		topPanel.setOptions("collapse_top", "'. $context['session_id'] .'");
		topPanel.addTogglePanel("upshrinkTopBar");

		var bottomPanel = new smfToggle("upshrinkBottom", '. (empty($options['collapse_bottom']) ? 'false' : 'true') .');
		bottomPanel.useCookie('. ($context['user']['is_guest'] ? 1 : 0) .');
		bottomPanel.setOptions("collapse_bottom", "'. $context['session_id'] .'");
		bottomPanel.addTogglePanel("upshrinkBottomBar");

		function setUpshrinkTitle(elm, Panel) {
			document.getElementById(elm).setAttribute("title", document.getElementById(elm).title.indexOf("'.$txt['pmx_show_index'].'") != -1 ? "'. $txt['pmx_hidepanel'] .'" + Panel : "'. $txt['pmx_showpanel'] .'" + Panel); }

		function setBlockUpshrinkTitle(elm, Panel) {
			document.getElementById(elm).setAttribute("title", document.getElementById(elm).title.indexOf("'.$txt['pmx_expand_index'].'") != -1 ? "'. $txt['pmx_collapse'] .'" + Panel : "'. $txt['pmx_expand'] .'" + Panel); }

		function Left_Panel() {
			if('. intval($context['PortaMx']['collapse']['left']) .' == 1) {
				leftPanel.toggle(); 
				setUpshrinkTitle(\'xbarleft\', \''. $txt['pmx_block_panels']['left'] .'\'); }
		}
		function Right_Panel() {
			if('. intval($context['PortaMx']['collapse']['right']) .' == 1) {
				rightPanel.toggle(); 
				setUpshrinkTitle(\'xbarright\', \''. $txt['pmx_block_panels']['right'] .'\'); }
		}
		function Top_Panel() {
			if('. intval($context['PortaMx']['collapse']['top']) .' == 1) {
				topPanel.toggle(); 
				setUpshrinkTitle(\'xbartop\', \''. $txt['pmx_block_panels']['top'] .'\'); }
		}
		function Bottom_Panel() {
			if('. intval($context['PortaMx']['collapse']['bottom']) .' == 1) {
				bottomPanel.toggle(); 
				setUpshrinkTitle(\'xbarbottom\', \''. $txt['pmx_block_panels']['bottom'] .'\'); }
		}
		function Info_Toggle(elm) {
			elm2 = elm + "2";
			elm2State = document.getElementById(elm2).style.display;
			document.getElementById(elm2).style.display = document.getElementById(elm).style.display;
			document.getElementById(elm).style.display = elm2State; }

		function ShowHTML(pageid)
		{
			var teasedid = "teasedhtml" + pageid;
			var fullid = "fullhtml" + pageid;
			if(document.getElementById(fullid).style.display == "none")
			{
				document.getElementById(teasedid).style.display = "none"
				document.getElementById(fullid).style.display = ""
			}
			else
			{
				document.getElementById(fullid).style.display = "none"
				document.getElementById(teasedid).style.display = ""
			}
		}

		function ResizeImages(ID, rescale)
		{
			var ImgData = eval("MsgImages"+ID);
			var setlen = ImgData.length -1;
			var elements = new Array();
			var elm = 0;
			var failed = 0;

			for(var idx = 0; idx < setlen; idx++)
			{
				for(var i = 0; i < document.getElementsByName(ImgData[idx]).length; i++)
				{
					if(document.getElementsByName(ImgData[idx])[i].src != "")
					{
						if(document.getElementsByName(ImgData[idx])[i].complete)
						{
							elements[elm] = document.getElementsByName(ImgData[idx])[i];
							elm++;
						}
						else
							failed++;
					}
				}
			}

			if(failed == 0)
			{
				window.clearInterval(ImgData[setlen]);
				var imgsrc = "";
				for(var idx = 0; idx < elements.length; idx++)
				{
					if(imgsrc != elements[idx].src)
					{
						imgsrc = elements[idx].src
						var Xsize = elements[idx].width;
						var Ysize = elements[idx].height;
					}
					var fact = ((Xsize > rescale || Ysize > rescale) ? (Xsize > Ysize ? Xsize : Ysize) : 1);
					elements[idx].width = Math.ceil(Xsize * (rescale / fact));
					elements[idx].height = Math.ceil(Ysize * (rescale / fact));
				}
			}
		}

		function languageChange(lang)
		{
			var curpage = new String(window.location);
			curpage = curpage.search(/\?/) != -1 ? curpage.substring(curpage.search(/\?/)) : "";
      curpage = curpage.replace(/\;/g, "|");
			var newlang = lang + ";pmxlangchg=" + escape(curpage);
			window.location.href = "'. $scripturl .'?language=" + newlang;
		}

		function xBarKeys(Events)
		{
			if(pmx_xBarKeys)
			{
				var xKey = 0;
				if(!Events)
					Events = window.event;
				if(Events.ctrlKey)
				{
					if(Events.which)
						xKey = Events.which;
					else
					{
						if(Events.keyCode)
							xKey = Events.keyCode;
					}
					switch(xKey)
					{
						case 37:
							Left_Panel();
							return false;
						case 38:
							Top_Panel();
							return false;
						case 39:
							Right_Panel();
							return false;
						case 40:
							Bottom_Panel();
							return false;
					}
				}
			}
		}
		document.onkeydown = xBarKeys;
	// ]]></script>
';
}
?>